@@ -1,5 +1,5 @@
 (* 
- * $Id: uq_engines.ml 2221 2015-02-23 22:19:42Z gerd $
+ * $Id$
  *)
 
 open Printf
@@ -13,7 +13,7 @@ exception Cancelled
 exception Timeout
 
 class type async_out_channel = object
-  method output : string -> int -> int -> int
+  method output : Bytes.t -> int -> int -> int
   method close_out : unit -> unit
   method pos_out : int
   method flush : unit -> unit
@@ -24,7 +24,7 @@ end
 
 
 class type async_in_channel = object
-  method input : string -> int -> int -> int
+  method input : Bytes.t -> int -> int -> int
   method close_in : unit -> unit
   method pos_in : int
   method can_input : bool
@@ -559,6 +559,11 @@ object(self)
 	| Some e -> 
 	    e # abort()
 	| None -> ()
+    );
+    ( match !proxy with
+	| Some e -> 
+	    e # abort()
+	| None -> ()
     )
 end;;
 
@@ -1285,7 +1290,7 @@ object
   method reading : bool
   method start_reading : 
     ?peek:(unit -> unit) ->
-    when_done:(exn option -> int -> unit) -> string -> int -> int -> unit
+    when_done:(exn option -> int -> unit) -> Bytes.t -> int -> int -> unit
   method start_mem_reading : 
     ?peek:(unit -> unit) ->
     when_done:(exn option -> int -> unit) -> Netsys_mem.memory -> int -> int ->
@@ -1293,7 +1298,7 @@ object
   method cancel_reading : unit -> unit
   method writing : bool
   method start_writing :
-    when_done:(exn option -> int -> unit) -> string -> int -> int -> unit
+    when_done:(exn option -> int -> unit) -> Bytes.t -> int -> int -> unit
   method start_mem_writing : 
     when_done:(exn option -> int -> unit) -> Netsys_mem.memory -> int -> int ->
     unit
@@ -1428,9 +1433,9 @@ class type wrapped_datagram_socket =
 object
   method descriptor : Unix.file_descr
   method sendto : 
-    string -> int -> int -> Unix.msg_flag list -> sockspec -> int
+    Bytes.t -> int -> int -> Unix.msg_flag list -> sockspec -> int
   method recvfrom : 
-    string -> int -> int -> Unix.msg_flag list -> (int * sockspec)
+    Bytes.t -> int -> int -> Unix.msg_flag list -> (int * sockspec)
   method shut_down : unit -> unit
   method datagram_type : datagram_type
   method socket_domain : Unix.socket_domain
