@@ -1,4 +1,4 @@
-(* $Id: netplex_kit.mli 1652 2011-08-03 21:50:30Z gerd $ *)
+(* $Id$ *)
 
 (** Netplex toolkit *)
 
@@ -138,6 +138,8 @@ val create_protocol : ?lstn_backlog:int ->
                       ?lstn_reuseaddr:bool ->
                       ?so_keepalive:bool -> 
                       ?tcp_nodelay:bool ->
+                      ?local_chmod:int ->
+                      ?local_chown:(int*int) ->
                       ?configure_slave_socket:(Unix.file_descr ->unit) ->
                       string ->
                       extended_address array ->
